#!/usr/bin/env node
'use strict';

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nodeElmCompiler = require('node-elm-compiler');

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const elmFilename = (process.argv[2] || '').trim();

if (!elmFilename || elmFilename === '-h' || elmFilename === '--help') {
  console.log('Usage: run-elm [FILENAME]');
  process.exit(1);
}

if (!_shelljs2.default.which('elm')) {
  console.error('Error: No elm global binary available. Please install with `npm install -g elm`.');
  process.exit(1);
}

const readFile = _util2.default.promisify(_fs2.default.readFile);
const writeFile = _util2.default.promisify(_fs2.default.writeFile);
const unlink = _util2.default.promisify(_fs2.default.unlink);

const runModuleName = 'RunElmMain';
const templatePath = _path2.default.resolve(__dirname, `${runModuleName}.elm.template`);
const templateWithArgsPath = _path2.default.resolve(__dirname, `${runModuleName}Args.elm.template`);

const moduleName = _path2.default.basename(elmFilename, '.elm');
const outputDir = _path2.default.resolve(_path2.default.dirname(elmFilename));
const outputElmFilename = _path2.default.join(outputDir, `${runModuleName}.elm`);
const outputCompiledFilename = _path2.default.join(outputDir, 'run-elm-main.js');

function getElmFileContents() {
  return readFile(elmFilename, 'utf-8').catch(() => Promise.reject(`Elm file '${elmFilename}' does not exist`));
}

function determineNeedArgs(elmFileContents) {
  const argsRegex = /^output *\w+ *=/;

  return elmFileContents.split('\n').some(line => argsRegex.test(line.trim()));
}

function getTemplate(needArgs) {
  const templateFile = needArgs ? templateWithArgsPath : templatePath;

  return readFile(templateFile, 'utf-8');
}

function createElmFile(template) {
  const contents = template.replace('{module}', moduleName);

  return writeFile(outputElmFilename, contents, 'utf-8');
}

async function compileElmFile() {
  _shelljs2.default.cd(outputDir);

  const contents = await (0, _nodeElmCompiler.compileToString)([outputElmFilename], { yes: true });

  return writeFile(outputCompiledFilename, contents);
}

function runElmFile(needArgs) {
  return new Promise(resolve => {
    const { worker } = require(outputCompiledFilename)[runModuleName];
    let app;

    if (needArgs) {
      const args = process.argv.slice(3);
      app = worker(args);
    } else {
      app = worker();
    }

    app.ports.sendOutput.subscribe(output => {
      console.log(output);
      resolve();
    });
  });
}

function cleanup() {
  return Promise.all([unlink(outputElmFilename).catch(() => {}), unlink(outputCompiledFilename).catch(() => {})]);
}

function handleError(err) {
  let message;

  if (typeof err === 'object' && 'message' in err) {
    if (/does not expose `output`/.test(err.message)) {
      message = `Elm module \`${moduleName}\` does not define a String \`output\`.`;
    } else {
      message = err.message;
    }
  } else {
    message = err;
  }

  console.error('Error:', message);
}

async function main() {
  try {
    const elmFileContents = await getElmFileContents();
    const needArgs = determineNeedArgs(elmFileContents);
    const template = await getTemplate(needArgs);

    await createElmFile(template);
    await compileElmFile();
    await runElmFile(needArgs);
    await cleanup();
  } catch (e) {
    await cleanup();
    handleError(e);
    process.exit(1);
  }
}

main();